var morganaMap = null;		  // global GMap2 object shown under Locations

var morganaGeocoder = null;	  // global GClientGeocoder for the address form
var morganaAddressMarker = null;  // GMarker for the address form

function renderLocation(loc) {
  var html = "";
  html += "<b>" + loc.name + "</b><br />\n";
  html += "<br />\n";
  html += "jump to: ";
  html += "<a href=\"" + loc.url + "\">Zenoss location</a><br /><br />\n";
  html += "manage: ";
  html += "<a href=\"" + loc.url + "/setGeoLocation?delete=true\">remove geographic info</a>";
  return html;
}

function placeLocation(map, loc, icon) {
  if (loc.status == 7) {  // location with no geographic information
    return;
  }
  var point = new GLatLng(loc.latitude, loc.longitude);
  var marker = new GMarker(point, icon);
  map.addOverlay(marker);
  GEvent.addListener(marker, "click", function() {
    marker.openInfoWindowHtml(renderLocation(loc));
  });
}

function setGeoLocation() {
  var sel = document.getElementById('morganaGeoLocationChoice').selectedIndex;
  var loc = locations[sel];
  var lat = document.getElementById('morganaLat').value;
  var lng = document.getElementById('morganaLng').value;
  var url = loc.url + "/setGeoLocation?lat=" + lat + "&long=" + lng;
  window.location = url;
}

function renderGeoLocationChoice(point) {
  var html = "";
  var chopPrefix = "/zport/dmd/Locations";  // prefix of the 'url' field of the
    // 'locations' record which should be chopped away to obtain a location path
  var chopLen = chopPrefix.length;

  html += "set <i>here</i> position of location ";
  html += "<input id='morganaLat' type='hidden' value='" + point.lat() + "'>\n";
  html += "<input id='morganaLng' type='hidden' value='" + point.lng() + "'>\n";
  html += "<select id='morganaGeoLocationChoice'>\n";
  for (var i = 0; i < locations.length; i++) {
    var url = locations[i].url;
    html += "<option";
    if (url == currentLocationUrl) {
      html += " selected='1'";
    }
    html += " value='" + i + "'>";
    html += url.substring(chopLen);
    html += "</option>\n";
  }
  html += "</select>\n";
  html += "<button onClick='setGeoLocation()'>set position</button>\n";
  return html;
}

function updateAddressMarker(point, html) {
  morganaAddressMarker.setPoint(point);
  if (morganaAddressMarker.isHidden()) {
    morganaAddressMarker.show()
  }
  morganaAddressMarker.openInfoWindowHtml(html);
  GEvent.clearListeners(morganaAddressMarker, "click");
  GEvent.addListener(morganaAddressMarker, "click", function () {
    morganaAddressMarker.openInfoWindowHtml(html);
  });
}

function searchAddress(address) {
  morganaGeocoder.getLatLng(
    address,
    function(point) {
	if (!point) {
	  alert(address + " not found");
	} else {
	  var html = "<b>Address:</b>" + address;
	  html += "<br /><br />\n";
	  html += renderGeoLocationChoice(point);
	  updateAddressMarker(point, html);
	  morganaMap.setCenter(point, 13);
	}
      }
    );
}

function loadGMap() {
  var icons = [
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-critical.png"),  // >= 1 critical
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-error.png"),     // >= 1 errors
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-warning.png"),   // >= 1 warnings
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-info.png"),	// >= 1 info
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-debug.png"),	// >= 1 debug
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-ok.png"),	// no errors
    new GIcon(G_DEFAULT_ICON, "pushpins/loc-idle.png"),      // idle location
      // no icon for status 7, those locations don't have geographic info, as
      // such they can't be shown on map
  ];
  if (GBrowserIsCompatible()) {
    morganaMap = new GMap2(document.getElementById("map"));
    morganaGeocoder = new GClientGeocoder();
    morganaMap.addControl(new GLargeMapControl());
    morganaMap.addControl(new GMapTypeControl());

    var bestCenter = null;
    var bestZoomLevel = 8;
    if (boundaries) {
      bestCenter = boundaries.getCenter();
      bestZoomLevel = morganaMap.getBoundsZoomLevel(boundaries) - 1;
      if (bestZoomLevel < 0) {
	bestZoomLevel = 0;
      }
    } else {
      bestCenter = new GLatLng(44.505566, 11.341667); // arbitrary default
    }
    morganaMap.setCenter(bestCenter, bestZoomLevel);

    locations.sort(function(l1, l2) {
      if (l1.name < l2.name) {
	return (-1);
      } else if (l1.name > l2.name) {
	return 1;
      } else {
	return 0;
      }
    });
    for (var i = 0; i < locations.length; i++) {
      placeLocation(morganaMap, locations[i], icons[locations[i].status]);
    }

    // initially hidden marker, will be governed by the geocoding form
    morganaAddressMarker = new GMarker(bestCenter);
    morganaMap.addOverlay(morganaAddressMarker);
    morganaAddressMarker.hide();

    GEvent.addListener(morganaMap, "singlerightclick",
      function(point, src, overlay) {
	var ll = morganaMap.fromContainerPixelToLatLng(point);
	var html = renderGeoLocationChoice(ll);
	updateAddressMarker(ll, html);
      }
    );
  }
}

function unloadGMap() {
  GUnload();
}

addLoadEvent(loadGMap);
