import string

# constants to be kept in sync with the indexes of the "icons" array of
# google-maps.js
CRITICAL = 0
ERROR = 1
WARNING = 2
INFO = 3
DEBUG = 4
OK = 5
IDLE = 6
HIDDEN = 7  # for locations with no available geographic info

def render_locs(var_name, active_locs, idle_locs, hidden_locs):
    def loc2js(loc, idle=False, hidden=False):
        """location -> javascript record snippt"""
        parts = loc.getPhysicalPath()
        path = string.join(parts, '/')

        # status is used as an index in the icons array of google-maps.js:
        #   0 -> idle location (not used)
        #   1 -> location ok, no errors to be shown graphically
        #   2 -> location with >= 1 un-acknowledged error (severity 'error')
        #   3 -> location with >= 1 un-acknowledged error (severity 'critical')
        status = 1
        summary = loc.getEventSummary()
        # sample return value of Zenoss' getEventSummary()
        # In [26]: loc.getEventSummary()
        # Out[26]: 
        # [['zenevents_5_noack noack', 0, 0],   # <- critical errors ack/unack
        #  ['zenevents_4_noack noack', 0, 0],   # <- errors ack/unack
        #  ['zenevents_3_noack noack', 0, 0],   # <- warnings ack/unack
        #  ['zenevents_2_noack noack', 2, 5],   # <- info ack/unack
        #  ['zenevents_1_noack noack', 0, 0]]   # <- debug ack/unack

        if hidden:
            status = HIDDEN
        elif idle:
            status = IDLE
        else:
            status = OK
            for i in range(0,5):
                if summary[i][2] - summary[i][1] > 0:
                    # >= 1 unacked error of severity i
                    status = i
                    break


        js = '{ url: "%s", name: "%s", status: %d' % \
                (string.join(parts, '/'), string.join(parts[4:], '/'), status)
        if status != HIDDEN:
            js += ', latitude: %f, longitude: %f' % \
                (loc.getProperty('latitude'), loc.getProperty('longitude'))
        js += ' }'
        return js

    text = 'var %s = [\n' % var_name
    rendered_locs = map(loc2js, active_locs)
    rendered_locs.extend(map((lambda l: loc2js(l, idle=True)), idle_locs))
    rendered_locs.extend(map((lambda l: loc2js(l, hidden=True)), hidden_locs))
    text += string.join(rendered_locs, ',\n')
    text += '];'

    return text

def render_bounds(var_name, bounds):
    text = 'var %s = ' % var_name
    if bounds:
        sw, ne = bounds
        text += 'new GLatLngBounds(new GLatLng(%f, %f), new GLatLng(%f, %f));'\
                % (sw[0], sw[1], ne[0], ne[1])
    else:
        text += 'null;'

    return text

def bounding_box(locs):
    """Given a location list return a pair of coordinates <SW, NE> identifying
    the minimum squared area which contains all locations. The return type is
    intuitively isomorphic to Google Maps' GLatLngBounds."""

    latitudes = map(lambda loc: loc.getProperty('latitude'), locs)
    longitudes = map(lambda loc: loc.getProperty('longitude'), locs)

    if latitudes != [] and longitudes != []:
        sw = min(latitudes), min(longitudes)
        ne = max(latitudes), max(longitudes)
        return (sw, ne)
    else:
        return None

cur_loc, active_locs, idle_locs, hidden_locs = context.collectGeoLocations()
bounds = bounding_box(active_locs)

print render_locs('locations', active_locs, idle_locs, hidden_locs)
if cur_loc:
    print 'var currentLocationUrl = "%s";\n' % \
        (string.join(cur_loc.getPhysicalPath(), '/'))
print render_bounds('boundaries', bounds)

return printed
